<?php

namespace ZapperIntegration\Payments\Controller\Payments;

use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Store\Model\StoreManagerInterface;
use ZapperIntegration\Payments\Model\UrlHelper;
use ZapperIntegration\Payments\Model\ZapperConstants;

class Checkout implements HttpGetActionInterface
{
    private Curl $curl;
    private ResultFactory $resultFactory;
    private Session $checkoutSession;
    private ScopeConfigInterface $scopeConfig;
    private StoreManagerInterface $storeManager;

    /**
     * @param Curl $curl
     * @param ResultFactory $resultFactory
     * @param Session $checkoutSession
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(Curl $curl, ResultFactory $resultFactory, Session $checkoutSession, ScopeConfigInterface $scopeConfig, StoreManagerInterface $storeManager)
    {
        $this->curl = $curl;
        $this->resultFactory = $resultFactory;
        $this->checkoutSession = $checkoutSession;
        $this->scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
    }

    /**
     * Execute action based on request and return result
     *
     * @return ResultInterface
     * @throws NoSuchEntityException
     */
    public function execute() : ResultInterface
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        $order = $this->checkoutSession->getLastRealOrder();

        $merchantId = $this->scopeConfig->getValue(ZapperConstants::CONFIG_MERCHANT_ID);
        $siteId = $this->scopeConfig->getValue(ZapperConstants::CONFIG_SITE_ID);

        $store = $this->storeManager->getStore();
        $baseUrl = $store->getBaseUrl();
        $storeName = $store->getName();

        $body = $this->createBody(
            $order->getBaseGrandTotal(),
            $merchantId,
            $siteId,
            $baseUrl,
            $order->getId(),
            $storeName
        );

        $this->curl->setTimeout(ZapperConstants::ECOMMERCE_GATEWAY_TIMEOUT_SECONDS);
        $this->curl->post($this->getUrl() . "magento", $body);

        if ($this->curl->getStatus() != 302) {
            $response->setUrl($body["cancelUrl"]);
            return $response;
        }

        $responseHeaders = $this->curl->getHeaders();
        $response->setUrl(
            UrlHelper::buildUrl(
                [$this->getUrl(), $responseHeaders["location"]]
            )
        );
        return $response;
    }

    private function getUrl() : string {
        return getenv(ZapperConstants::ENV_VAR_ZAPPER_ECOM_GATEWAY) ?: ZapperConstants::ECOMMERCE_GATEWAY_URL;
    }

    private function createBody(
        $amount,
        $merchant,
        $site,
        $baseUrl,
        $orderId,
        $name
    ) : array {
        return [
            "amount" => $amount,
            "merchantId" => $merchant,
            "siteId" => $site,
            "notificationUrl" => UrlHelper::buildUrl([$baseUrl, ZapperConstants::API_NOTIFICATION]),
            "cancelUrl" =>  UrlHelper::buildUrl([$baseUrl, ZapperConstants::API_CANCEL]),
            "completeUrl" =>  sprintf("%s?orderId=%s", UrlHelper::buildUrl([$baseUrl, ZapperConstants::API_COMPLETE]), $orderId),
            "reference" => $orderId,
            "siteName" => $name
        ];
    }
}
